# -*- coding: utf-8 -*-
#
# This is the main application controls
# and the renaming functions.
#
"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

# needed modules:
from __future__ import print_function

import wx
import sys
import platform
import os
import codecs
import re
import time
import wx.lib.dialogs
import EnhancedStatusBar as ESB
import gettext
import locale
import getopt

# notebok windows:
import sorting
import errors

# dialogs:
import preferences
import helpDiag
import about
import langSelect

# utilities
import utils
import classes

def create(parent):
    return MainWindow(parent)


[wxID_MAIN_WINDOW, wxID_MAIN_WINDOWDISPLAY,
wxID_MAIN_WINDOWNOTEBOOK, wxID_MAIN_WINDOWSTATUSBAR1, wxID_MAIN_WINDOWSTATUSIMAGE,
 wxID_SELECTIONAREA,

wxID_MENURENAMER_DESTROY, wxID_MENURENAMER_DESTROYALL,

wxID_MENUHELP_ABOUT, wxID_MENUHELP_HELP,
wxID_MENUHELP_EXAMPLES, wxID_MENUHELP_REHELP,
wxID_MENUHELP_FORMATHELP,

wxID_MENUFILE_EXIT, wxID_MENUFILE_PREVIEW,
wxID_MENUFILE_GO, wxID_MENUFILE_LOADINI,
wxID_MENUFILE_SAVEINI, wxID_MENUFILE_SAVELOG,
wxID_MENUFILE_IMPORT,

wxID_MENUPICKER_ALL, wxID_MENUPICKER_NONE,
wxID_MENUPICKER_WALK, wxID_MENUPICKER_BROWSE,
wxID_MENUPICKER_OK,

wxID_MENUSETTINGS_PREFERENCES, wxID_MENUSETTINGS_LANG,

] = [wx.NewId() for initCollMenuEditItems in range(27)]


# M main splitter
class MySplitter(wx.SplitterWindow):
    def __init__(self, parent, ID):
        wx.SplitterWindow.__init__(self, parent, ID,
                        style=wx.SP_LIVE_UPDATE|wx.SP_3DSASH|wx.SP_NO_XP_THEME)


# Main application class:
class MainWindow(wx.Frame):

    def initCollMenuBarMenus(self, parent):
        parent.Append(menu=self.menuFile, title=_(u"File"))
        parent.Append(menu=self.menuPicker, title=_(u"Picker"))
        parent.Append(menu=self.menuRenamer, title=_(u"Renamer"))
        parent.Append(menu=self.menuEdit, title=_(u"Settings"))
        parent.Append(menu=self.menuHelp, title=_(u"Help"))
        self.menuPicker.getAllMenu.Enable(False)
        self.menuPicker.getNoneMenu.Enable(False)
        self.menuFile.GoMenu.Enable(False)


    def initCollMenuFileItems(self, parent):
        # FILE MENU
        parent.LoadIniMenu = wx.MenuItem(parent, wxID_MENUFILE_LOADINI,
            _(u"&Load Config\tctrl+L"),
            self.makeSpace(_(u"Load configuration from file")))
        parent.LoadIniMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'loadIni.png'),
            wx.BITMAP_TYPE_PNG))
        parent.SaveIniMenu = wx.MenuItem(parent, wxID_MENUFILE_SAVEINI,
            _(u"&Save Config\tctrl+S"),
            self.makeSpace(_(u"Save configuration to file")))
        parent.SaveIniMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'saveIni.png'),
            wx.BITMAP_TYPE_PNG))

        parent.SaveLog = wx.MenuItem(parent, wxID_MENUFILE_SAVELOG,
            _(u"&Export to log file"),
            self.makeSpace(_(u"Save current snapshot to a log file")))
        parent.SaveLog.SetBitmap(wx.Bitmap(utils.iconPath(u'CSVto.png'),
            wx.BITMAP_TYPE_PNG))

        parent.Import = wx.MenuItem(parent,
         wxID_MENUFILE_IMPORT, _(u"&Import from log file"),
         self.makeSpace(_(u"Load a snapshot")))
        parent.Import.SetBitmap(wx.Bitmap(utils.iconPath(u'CSVfrom.png'),
            wx.BITMAP_TYPE_PNG))

        parent.PreviewMenu = wx.MenuItem(parent, wxID_MENUFILE_PREVIEW,
            _(u"&Preview\tF7"), self.makeSpace(_(u"Preview selection")))
        parent.PreviewMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'preview.png'),
            wx.BITMAP_TYPE_PNG))
            
        parent.GoMenu = wx.MenuItem(parent, wxID_MENUFILE_GO,
            _(u"&Go !\tF8"), self.makeSpace(_(u"Rename selection")))
        parent.GoMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'go.png'),
            wx.BITMAP_TYPE_PNG))

        parent.exitMenu = wx.MenuItem(parent, wxID_MENUFILE_EXIT,
            _(u"&Quit\tctrl+Q"), self.makeSpace(_(u"Quit Metamorphose")))
        parent.exitMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'exit.png'),
            wx.BITMAP_TYPE_PNG))

        parent.AppendItem(parent.LoadIniMenu)
        parent.AppendItem(parent.SaveIniMenu)
        parent.AppendSeparator()
        parent.AppendItem(parent.SaveLog)
        parent.SaveLog.Enable(False)
        parent.AppendItem(parent.Import)
        parent.AppendSeparator()
        parent.AppendItem(parent.PreviewMenu)
        parent.AppendItem(parent.GoMenu)
        parent.AppendSeparator()
        parent.AppendItem(parent.exitMenu)

        self.Bind(wx.EVT_MENU, self.saveAsText,
              id=wxID_MENUFILE_SAVELOG)
        self.Bind(wx.EVT_MENU, self.importSelection,
              id=wxID_MENUFILE_IMPORT)
        self.Bind(wx.EVT_MENU, self.saveCfg,
              id=wxID_MENUFILE_SAVEINI)
        self.Bind(wx.EVT_MENU, self.loadCfg,
              id=wxID_MENUFILE_LOADINI)
        self.Bind(wx.EVT_MENU, self.onPreviewButton,
              id=wxID_MENUFILE_PREVIEW)
        self.Bind(wx.EVT_MENU, self.renameItems,
              id=wxID_MENUFILE_GO)
        self.Bind(wx.EVT_MENU, self.onMenuFileExitMenu,
              id=wxID_MENUFILE_EXIT)


    def initCollMenuRenamerItems(self, parent):
        parent.destroyMenu = wx.MenuItem(parent, wxID_MENURENAMER_DESTROY,
            _(u"Destroy operation"),
            self.makeSpace(_(u"Destroy visible operation")))
        parent.destroyMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'errors.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.destroyAllMenu = wx.MenuItem(parent, wxID_MENURENAMER_DESTROYALL,
            _(u"Destroy all operations\tctrl+D"),
            self.makeSpace(_(u"Destroy all operations")))
        parent.destroyAllMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'nuke.png'),
            wx.BITMAP_TYPE_PNG))

        parent.AppendItem(parent.destroyMenu)
        parent.AppendItem(parent.destroyAllMenu)

        self.Bind(wx.EVT_MENU, self.renamer.Panel.deleteOperation,
              id=wxID_MENURENAMER_DESTROY)
        self.Bind(wx.EVT_MENU, self.renamer.Panel.destroyAllGUIOperations,
              id=wxID_MENURENAMER_DESTROYALL)


    def initCollMenuPickerItems(self, parent):
        parent.browseMenu = wx.MenuItem(parent, wxID_MENUPICKER_BROWSE,
            _(u"&Browse...\tF4"),
            self.makeSpace(_(u"Browse for path")))
        parent.browseMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'browse.png'), wx.BITMAP_TYPE_PNG))
        parent.okMenu = wx.MenuItem(parent, wxID_MENUPICKER_OK,
            _(u"&Refresh\tF5"),
            self.makeSpace(_(u"Load or reload current path")))
        parent.okMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'reload.png'),
            wx.BITMAP_TYPE_PNG))

        parent.getAllMenu = wx.MenuItem(parent, wxID_MENUPICKER_ALL,
            _(u"Select &All\tctrl+A"),
            self.makeSpace(_(u"Select all items in picker")))
        parent.getAllMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'selectAll.png'), wx.BITMAP_TYPE_PNG))
        parent.getNoneMenu = wx.MenuItem(parent, wxID_MENUPICKER_NONE,
            _(u"Select &None\tctrl+N"),
            self.makeSpace(_(u"Deselect all items in picker")))
        parent.getNoneMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'selectNone.png'), wx.BITMAP_TYPE_PNG))
        parent.walkMenu = wx.MenuItem(parent, wxID_MENUPICKER_WALK,
            _(u"Recursive &selection\tctrl+R"),
            self.makeSpace(_(u"Get all files in directory and sub-directories, but no folders")))
        parent.walkMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'walk.png'),
            wx.BITMAP_TYPE_PNG))

        parent.AppendItem(parent.browseMenu)
        parent.AppendItem(parent.okMenu)
        parent.AppendSeparator()
        parent.AppendItem(parent.getAllMenu)
        parent.AppendItem(parent.getNoneMenu)
        parent.AppendItem(parent.walkMenu)

        self.Bind(wx.EVT_MENU, self.picker.browseForPath,
              id=wxID_MENUPICKER_BROWSE)
        self.Bind(wx.EVT_MENU, self.picker.setPanelPath,
              id=wxID_MENUPICKER_OK)
        self.Bind(wx.EVT_MENU, self.picker.selectNoItemsInPanel,
              id=wxID_MENUPICKER_NONE)
        self.Bind(wx.EVT_MENU, self.picker.selectAllItemsInPanel,
              id=wxID_MENUPICKER_ALL)
        self.Bind(wx.EVT_MENU, self.picker.walkFromMenu,
              id=wxID_MENUPICKER_WALK)


    def initCollMenuEditItems(self, parent):
        parent.PrefsMenu = wx.MenuItem(parent,
         wxID_MENUSETTINGS_PREFERENCES, _(u"Preferences"),
         self.makeSpace(_(u"Change your preferences")))

        parent.langMenu = wx.MenuItem(parent,
         wxID_MENUSETTINGS_LANG, _(u"Language"),
         self.makeSpace(_(u"Change the language")))

        parent.PrefsMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'preferences.ico'), wx.BITMAP_TYPE_ICO))
        parent.AppendItem(parent.PrefsMenu)

        parent.langMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'language.png'),wx.BITMAP_TYPE_PNG))
        parent.AppendItem(parent.langMenu)

        self.Bind(wx.EVT_MENU, self.onMenuEditPreferencesMenu,
              id=wxID_MENUSETTINGS_PREFERENCES)
        self.Bind(wx.EVT_MENU, self.languageSelect,
              id=wxID_MENUSETTINGS_LANG)


    def initCollMenuHelpItems(self, parent):
        parent.aboutMenu = wx.MenuItem(parent,
         wxID_MENUHELP_ABOUT, _(u"About"),
         self.makeSpace(_(u"Display general information about Metamorphose")))
        parent.aboutMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'about.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.helpMenu = wx.MenuItem(parent,
         wxID_MENUHELP_HELP, _(u"&Help\tF1"),
         self.makeSpace(_(u"How to use Metamorphose")))
        parent.helpMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'help.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.examplesMenu = wx.MenuItem(parent,
         wxID_MENUHELP_EXAMPLES, _(u"&Examples\tF2"),
         self.makeSpace(_(u"Some useful examples")))
        parent.examplesMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'examples.ico'),wx.BITMAP_TYPE_ICO))

        parent.FormatHelpMenu = wx.MenuItem(parent,
         wxID_MENUHELP_FORMATHELP, _(u"&Date && Time Formats"),
         self.makeSpace(_(u"Display a reference for Date & Time formats")))
        parent.FormatHelpMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'date_time.ico'),wx.BITMAP_TYPE_ICO))

        parent.REhelpMenu = wx.MenuItem(parent,
         wxID_MENUHELP_REHELP, _(u"&Regular Expressions"),
         self.makeSpace(_(u"Display a regular expression reference")))
        parent.REhelpMenu.SetBitmap(wx.Bitmap(utils.iconPath(u're.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.AppendItem(parent.aboutMenu)
        parent.AppendItem(parent.helpMenu)
        parent.AppendItem(parent.examplesMenu)
        parent.AppendItem(parent.FormatHelpMenu)
        parent.AppendItem(parent.REhelpMenu)

        self.Bind(wx.EVT_MENU, self.onMenuHelpAboutMenu,
              id=wxID_MENUHELP_ABOUT)
        self.Bind(wx.EVT_MENU, self.onMenuHelpHelpMenu,
              id=wxID_MENUHELP_HELP)
        self.Bind(wx.EVT_MENU, self.showSmallHelp,
              id=wxID_MENUHELP_EXAMPLES)
        self.Bind(wx.EVT_MENU, self.showSmallHelp,
              id=wxID_MENUHELP_FORMATHELP)
        self.Bind(wx.EVT_MENU, self.showSmallHelp,
              id=wxID_MENUHELP_REHELP)


    def initNotebook(self):
        parent = self.notebook

        # init Core classes
        self.picker = picker.Core(parent, self)
        self.sorting = sorting.Core(parent, self)
        self.errors = errors.Core(parent, self)
        self.renamer = renamer.Core(parent, self)

        # add notebook pages to notebook
        parent.AddPage(self.picker.Panel, _(u"Picker"))
        parent.AddPage(self.renamer.Panel, _(u"- Renamer -"))
        parent.AddPage(self.sorting.Panel, _(u"Sorting"))
        parent.AddPage(self.errors.Panel, _(u"Errors/Warnings: %s")%0)

        # list containing notebook images
        il = wx.ImageList(16, 16)
        img0 = il.Add(wx.Bitmap(utils.iconPath(u'picker.ico'),
                      wx.BITMAP_TYPE_ICO))
        img1 = il.Add(wx.Bitmap(utils.iconPath(u'main.ico'),
                      wx.BITMAP_TYPE_ICO))
        img2 = il.Add(wx.Bitmap(utils.iconPath(u'sorting.ico'),
                      wx.BITMAP_TYPE_ICO))
        img3 = il.Add(wx.Bitmap(utils.iconPath(u'errors.ico'),
                      wx.BITMAP_TYPE_ICO))

        # set images to pages
        parent.AssignImageList(il)
        parent.SetPageImage(0, img0)
        parent.SetPageImage(1, img1)
        parent.SetPageImage(2, img2)
        parent.SetPageImage(3, img3)
        # required to make XP theme colours ok
        self.notebook.SetBackgroundColour(self.notebook.GetThemeBackgroundColour())

    def sizer(self):
        mainSizer = self.mainSizer = wx.BoxSizer(wx.VERTICAL)
        mainSizer.Add(self.splitter,1,wx.EXPAND|wx.ALL, 5)
        self.SetSizerAndFit(mainSizer)

    def initUtils(self):
        self.menuFile = wx.Menu()
        self.menuPicker = wx.Menu()
        self.menuEdit = wx.Menu()
        self.menuHelp = wx.Menu()
        self.menuBar = wx.MenuBar()
        self.menuRenamer = wx.Menu()
        self.initCollMenuFileItems(self.menuFile)
        self.initCollMenuPickerItems(self.menuPicker)
        self.initCollMenuRenamerItems(self.menuRenamer)
        self.initCollMenuEditItems(self.menuEdit)
        self.initCollMenuHelpItems(self.menuHelp)
        self.initCollMenuBarMenus(self.menuBar)
        self.SetMenuBar(self.menuBar)

    # get fonts from system or specify own
    def initFonts(self):
        if wx.Platform == '__WXGTK__':
            sysFont = wx.SystemSettings.GetFont(wx.SYS_SYSTEM_FONT)
            self.fontParams = {
                'size' : sysFont.GetPixelSize()[0],
                'style' : sysFont.GetStyle(),
                'family' : sysFont.GetFamily(),
                'weight' : sysFont.GetWeight(),
            }
        else:
            self.fontParams = {
                'size' : 9,
                'style' : wx.NORMAL,
                'family' : wx.DEFAULT,
                'weight' : wx.NORMAL,
            }
        #print self.fontParams
        self.SetFont(wx.Font(
            self.fontParams['size'],
            self.fontParams['family'],
            self.fontParams['style'],
            self.fontParams['weight'])
        )

    def initCtrls(self, prnt):
        if self.debug:
            self.SetTitle(u"Métamorphose 2 v. %s -- DEBUG MODE"%self.version)
        else:
            self.SetTitle(u"Métamorphose 2 (beta)")
        self.SetBackgroundColour(wx.NullColour)
        self.SetAutoLayout(False)
        self.SetStatusBarPane(0)
        self.SetIcon(wx.Icon(utils.iconPath(u'metamorphose.ico'), wx.BITMAP_TYPE_ICO))
        self.SetThemeEnabled(True)
        self.initFonts()

        self.statusBar1 = ESB.EnhancedStatusBar(id=wxID_MAIN_WINDOWSTATUSBAR1,
              name=u'statusBar1', parent=self)
        self.SetStatusBar(self.statusBar1)

        self.statusImage = wx.StaticBitmap(bitmap=self.statusImages[u'eyes'],
              id=wxID_MAIN_WINDOWSTATUSIMAGE, name=u'statusImage',
              parent=self.statusBar1, size=wx.Size(-1, 16), style=0)
        self.statusBar1.AddWidget(self.statusImage, ESB.ESB_ALIGN_LEFT)

        self.splitter = MySplitter(self, wxID_SELECTIONAREA)

        # notebook
        self.notebook = wx.Notebook(id=wxID_MAIN_WINDOWNOTEBOOK,
              name=u'notebook', parent=self.splitter,
              style=wx.NB_TOP|wx.NO_BORDER)
        self.notebook.SetThemeEnabled(True)

        self.bottomWindow = bottomWindow.MainPanel(self.splitter, self)
        self.SetMinSize(wx.Size(-1,600))

        self.splitter.SetMinimumPaneSize(40)

        if wx.Platform == '__WXGTK__':
            split = -210
        elif wx.Platform == '__WXMSW__':
            split = -200
        else:
            split = -205
        self.splitter.SplitHorizontally(self.notebook, self.bottomWindow, split)


    def usage(self):
        print()
        print("Metamorphose 2.%s\nRunning on Python %s, wxPython %s"%(self.version, platform.python_version(), utils.getWxVersion()))
        print("Copyright (C) 2006-2010 ianare sevi")
        print("<http://file-folder-ren.sourceforge.net/>\n")
        print("Metamorphose is a graphical mass renaming program for files and folders.")
        print("There is support for automating GUI tasks via command line.\n")
        print("metamorphose2 [/path/to/open/spaces ok]")
        print("metamorphose2 [-t] [-d] [-p /path/to/open] [[-c /path/to/file.cfg] [-a level (0-3)]] [-l language]")
        print()
        print("-h,  --help       Show help screen and exit.")
        print("-t,  --timer      Show time taken to complete operations.")
        print("-d,  --debug      Show debugging information.")
        print("-p=, --path       Specify a directory to load.")
        print("-c=, --config     Specify a configuration file to load.")
        print("-a=, --auto       Specify automatic mode level to use with configuration file :")
        print("                    0 = do not preview items")
        print("                    1 = auto preview items")
        print("                    2 = auto rename items")
        print("                    3 = auto rename items and exit on success")
        print("-l=, --language  Overide prefered language :")
        print("                    en_US")
        print("                    fr")
        print()
        print("If no other options are given, you may specify a path to open :")
        print("$ metamorphose2 /srv/samba/Windows Likes Spaces")
        print()
        sys.exit()


    # Display message and exit
    def exit(self, message):
        print("Command line error :")
        print(message)
        print()
        sys.exit()


    # remove a leading '=' from CLI options
    def stripLeading(self, a):
        if '=' in a:
            a = a.lstrip('=')
        return a


    # Get options passed in at the command line
    def getCommandLineOptions(self):
        shortOptions = "htdp:vc:va:vl:v"
        longOptions = ["help", "timer", "debug", "path=", "config=", "auto=", "language="]

        try:
            opts, args = getopt.getopt(sys.argv[1:], shortOptions, longOptions)
        except getopt.GetoptError as err:
            # print help information and exit:
            print(str(err))
            sys.exit(2)

        # set options
        path = False
        configFilePath = False

        if sys.argv[1:] != [] and opts == []:
            path = ''
            for chunk in sys.argv[1:]:
                path += chunk + ' '

        for o, a in opts:
            # show processing times
            if o in ("-t", "--timer"):
                self.showTimes = True
                print("Showing processing times")
            # show debug messages
            elif o in ("-d", "--debug"):
                self.debug = True
                print("Running in debug mode")
                print("Version : " + self.version)
                print("Python %s, wxPython %s"%(platform.python_version(), utils.getWxVersion()))
            # set path
            elif o in ("-p", "--path"):
                path = self.stripLeading(a)
            # show help and exit
            elif o in ("-h", "--help"):
                self.usage()
            # use a config path
            elif o in ("-c", "--config"):
                configFilePath = self.stripLeading(a)
           # set auto level
            elif o in ("-a", "--auto"):
                level = self.stripLeading(a)
                if level not in ('0','1','2','3'):
                    exit("Invalid auto mode level : '%s'"%level)
                elif not configFilePath:
                    exit("Auto mode level must be used with configuration file ( '-c' or '--config' option )")
                else:
                    print("Auto mode level set : %s"%level)
                    self.autoModeLevel = int(level)
           # specify the language
            elif o in ("-l", "--language"):
                self.language = self.stripLeading(a)

        return path, configFilePath


    """
    Determine language to be loaded depending on : setting file, CLI option,
    or neither.
    Apply language to interface.
    """
    def setLanguage(self):
        # get language from file if not specified from command line
        if self.language == '':
            try:# see if language file exist
                langIni = codecs.open(utils.stdPath(u'language.ini'),'r', 'utf-8')
            except IOError:# have user choose language
                language = self.languageSelect(0)
            else:# get language from file
                language = langIni.read().strip()
        else:
            language = self.language

        # reference:
        locales = {
            #u'ar' : (wx.LANGUAGE_ARABIC, u'ar_SA.UTF-8'),
            #u'de' : (wx.LANGUAGE_GERMAN, u'de_DE.UTF-8'),
            #u'el' : (wx.LANGUAGE_GREEK, u'el_GR.UTF-8'),
            #u'en_GB' : (wx.LANGUAGE_ENGLISH_UK, u'en_GB.UTF-8'),
            u'en_US' : (wx.LANGUAGE_ENGLISH_US, u'en_US.UTF-8'),
            u'es' : (wx.LANGUAGE_SPANISH, u'es_ES.UTF-8'),
            u'fr' : (wx.LANGUAGE_FRENCH, u'fr_FR.UTF-8'),
            #u'he' : (wx.LANGUAGE_HEBREW, u'he_IL.UTF-8'),
            #u'hu' : (wx.LANGUAGE_HUNGARIAN, u'hu_HU.UTF-8'),
            #u'it' : (wx.LANGUAGE_ITALIAN, u'it_IT.UTF-8'),
            #u'ja' : (wx.LANGUAGE_JAPANESE, u'ja_JP.UTF-8'),
            #u'pl' : (wx.LANGUAGE_POLISH, u'pl_PL.UTF-8'),
            #u'pt_BR' : (wx.LANGUAGE_PORTUGUESE_BRAZILIAN, u'pt_BR.UTF-8'),
            #u'ru' : (wx.LANGUAGE_RUSSIAN, u'ru_RU.UTF-8'),
            #u'sv' : (wx.LANGUAGE_SWEDISH, u'sv_SE.UTF-8'),
            #u'tr' : (wx.LANGUAGE_TURKISH, u'tr_TR.UTF-8'),
            #u'zh_CN' : (wx.LANGUAGE_CHINESE_SIMPLIFIED, u'zh_CN.UTF-8'),
        }

        # right-to-left languages:
        rightToLeftLanguages = ('ar','dv','fa','ha','he','ps','ur','yi')

        try:
            locales[language]
        except KeyError:
            msg = u"Could not initialise language: '%s'.\nContinuing in "%language
            msg += u"American English (en_US)\n"
            print(msg)
            language = 'en_US'

        # needed for calendar and other things, send all logs to stderr
        wx.Log.SetActiveTarget(wx.LogStderr())

        # set locale and language
        wx.Locale(locales[language][0], wx.LOCALE_LOAD_DEFAULT)
        Lang = gettext.translation(u'metamorphose2', utils.localePath(language),
                                   languages=[locales[language][1]])
        Lang.install(unicode=True)

        # set some globals
        if language not in rightToLeftLanguages:
            self.langLTR = True
            self.alignment = wx.ALIGN_LEFT
        else:
            self.langLTR = False
            self.alignment = wx.ALIGN_RIGHT
        self.language = language
        self.encoding = unicode(locale.getlocale()[1])

        # to get some language settings to display properly:
        if platform.system() in ('Linux', 'FreeBSD'):
            try:
                os.environ['LANG'] = locales[language][1]
            except (ValueError, KeyError):
                pass


    def __init__(self, prnt):
        # Important variables needed throughout the application classes
        #
        self.version = utils.getVersion()
        self.warn = [] # warnings
        self.bad = [] # errors
        self.errorLog = [] # all errors go here
        self.items = [] # items to rename
        self.spacer = u" "*6 # spacer for status messages (to clear image)
        self.REmsg = False # regular expression error
        self.recursiveFolderOn = False # if true, will need to sort items before renaming
        self.language = '' # to be set later
        self.prefs = {} # preferences, to be set later
        self.autoModeLevel = False # automatic mode level
        self.showTimes = False # show processing times
        self.debug = False # show debug info

        path, configFilePath = self.getCommandLineOptions()

        wx.Frame.__init__(self, id=wxID_MAIN_WINDOW, name=u'MainWindow',
              parent=prnt, pos=wx.Point(-1, -1), size=wx.Size(-1, -1),
              style=wx.DEFAULT_FRAME_STYLE)

        # first run?
        utils.makeInitialEnvironment()

        self.setLanguage()

        if self.debug:
            print("Operating System :", platform.system())
            print("System encoding :", self.encoding)
            print("Interface language :", self.language)


        # import these modules here since they need language settings activated
        global renamer
        import renamer
        global configs
        import configs
        global picker
        import picker
        global bottomWindow
        import bottomWindow

        # initialise preferences
        self.prefs = preferences.Methods(self)

        # icons used for status bar messages
        self.statusImages = {
            u'failed' : wx.Bitmap(utils.iconPath(u'failed_sb.ico'),
                        wx.BITMAP_TYPE_ICO),
            u'wait' : wx.Bitmap(utils.iconPath(u'wait.png'),
                        wx.BITMAP_TYPE_PNG),
            u'warn' : wx.Bitmap(utils.iconPath(u'warn_sb.ico'),
                        wx.BITMAP_TYPE_ICO),
            u'complete' : wx.Bitmap(utils.iconPath(u'complete.ico'),
                        wx.BITMAP_TYPE_ICO),
            u'eyes' : wx.Bitmap(utils.iconPath(u'eyes.png'),
                        wx.BITMAP_TYPE_PNG),
            }

        # build main GUI
        self.initCtrls(prnt)

        # clear undo (if set in preferences):
        if self.prefs.get(u'clearUndo'):
            try:
                originalFile = codecs.open(utils.stdPath(u'undo/original.bak'),
                                           'w', "utf-8")
                originalFile.write('')
                renamedFile = codecs.open(utils.stdPath(u'undo/renamed.bak'),
                                          'w', "utf-8")
                renamedFile.write('')
            except IOError, error:
                utils.makeErrMsg(_(u"%s\n\nCould not clear undo")%error,
                                 _(u"Error"))
                pass

        # construct rest of GUI:
        self.initNotebook()
        self.initUtils()
        self.sizer()
        # call this after sizer to place properly:
        self.Center(wx.HORIZONTAL|wx.VERTICAL)

        # Load config from command line
        if configFilePath:
            configs.loadConfig(self, configFilePath)

        # Set root directory from command line arguments:
        if path:
            path = path.rstrip()
            self.picker.Panel.path.SetValue(path)
            self.picker.setPanelPath(True)


##### MISC STUFF: ##############################################################
    # Little functions to make repetative stuff easier to implement.

    # add spacing for translation
    def makeSpace(self,statusText):
        #if self.langLTR:
        #    return self.spacer+statusText
        #else:
        return self.spacer+statusText

    # this function is called by basically every widget in the app.
    # It calls the preview function, if the user has set so in preferences:
    def showPreview(self, event):
        if event and self.bottomWindow.autoPreview.GetValue():
            self.onPreviewButton(False)


    # open selection dialog for language:
    def languageSelect(self, event):
        # if triggered from event, language has already been loaded,
        # and we will be changing it. No event = no language file
        if event == 0:
            Title = u'\m/ (>_<) \m/'
        else:
            Title = _(u"Language")
            event = self.language

        # show the language choices
        dlg = langSelect.create(self, Title, event)
        if dlg.ShowModal() == wx.ID_OK:
            language = dlg.GetLanguage()
            dlg.Destroy()
            if self.language != language:
                self.changeLanguage(language, event)
            return language

    # write language.ini file
    def changeLanguage(self, language, event):
        try:
            langFile = codecs.open(utils.stdPath(u'language.ini'),'w', 'utf-8')
        except IOError, error:
            utils.makeErrMsg(unicode(error), u"Error")
            pass
        else:
            langFile.write(language)
            langFile.close()
        if event:
            msg = _(u"\n\nYou will need to restart Metamorphose to change the language.")
            msg += _(u"\n\nClose out of Metamorphose now?")
            msg = language + msg
            title = _(u"Change the Language")
            # Restart app automatically if not in windows
            if utils.makeYesNoMsg(msg, title):
                if platform.system() != 'Windows':
                    if wx.Process.Open(utils.realPath(sys.argv[0])):
                        self.Close()
                else:
                    self.Close()

    # Set status bar text and image:
    def setStatusMsg(self, msg, select):
        self.statusImage.SetBitmap(self.statusImages[select])
        self.SetStatusText(self.makeSpace(msg))

        if self.debug:
            print(u"status message : '%s'" %msg)



##### MENU ACTIONS: ############################################################
    def onMenuFileExitMenu(self, event):
        self.Close()

    def saveCfg(self, event):
        configs.saveConfig(self)

    def loadCfg(self, event):
        configs.loadConfig(self)

    #------- export current selection as CSV file -#
    def saveAsText(self, event):
        if hasattr(self,'toRename'):
            CSVfile = ""
            q = self.prefs.get('logEnclose')
            sep = self.prefs.get('logSeparator')
            ext = self.prefs.get('logFextension')

            if self.showTimes:
                t = time.time()

            # create file contents
            for original, renamed in self.toRename:
                CSVfile += unicode(q+original[0]+q+sep+q+renamed[0]+q+'\n')

            if self.showTimes:
                print("Export file contents for %s items : %s"%(len(self.toRename), (time.time() - t)))

            # trigered by menu, allow to choose output file
            if event:
                dlg = wx.FileDialog(self, message=_(u"Save current items as ..."),
                      defaultDir='', defaultFile=u'',
                      wildcard=_(u"Log file (*.%s)")%ext + u'|*.%s'%ext,
                      style=wx.SAVE|wx.OVERWRITE_PROMPT
                      )
                if dlg.ShowModal() == wx.ID_OK:
                    # attempt to write file
                    utils.writeFile(dlg.GetPath(), CSVfile)
                dlg.Destroy()
            # auto logging, generate file name
            else:
                file = time.strftime("undo_%Y-%m-%d_%Hh%Mm%Ss",
                       time.localtime())+'.'+self.prefs.get('logFextension')
                path = os.path.join(self.prefs.get(u'logLocation'), file)
                utils.writeFile(path, CSVfile)



    #------- imports selection from txt file -#
    def importSelection(self, event):
        if self.prefs.get('logLocation') != '':
            path = self.prefs.get('logLocation')
        else:
            path = utils.stdPath(u'undo')

        # settings from preferences
        ext = self.prefs.get('logFextension')
        q = self.prefs.get('logEnclose')
        sep = self.prefs.get('logSeparator')

        wildOnes = _(u"Log file") + u" (*.%s)|*.%s|"%(ext,ext) +\
                   _(u"All files") + "(*.*)|*.*"

        # file open dialog
        dlg = wx.FileDialog(
            self, message=_(u"Import selection from ..."), defaultDir=path,
            defaultFile=u'', wildcard=wildOnes,
            style=wx.OPEN|wx.CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            original = []
            renamed = []
            unloadbles = ""
            f = dlg.GetPath()

            # undo files have order reversed
            if os.path.basename(f)[0:5] == 'undo_':
                o = 1
                r = 0
            else:
                o = 0
                r = 1

            f = codecs.open(f, 'r', 'utf_8')
            i = 0
            for line in f.readlines():
                i += 1
                line = line.lstrip(q).rstrip(q+'\n').split(q+sep+q)
                # check for existance of every item
                if os.path.exists(line[o]):
                    original.append((line[o],1))
                    renamed.append([line[r],False])
                else:
                    unloadbles += "%s\n"%line[o]

            # set up for display
            self.toRename = zip(original,renamed)
            self.picker.Panel.path.SetValue('')
            self.picker.clearALL()

            # only allow rename if there are items
            if len(self.toRename) != i:
                msg = _(u"These items could not be loaded, it doesn't look like they exist :\n\n")
                msg += unloadbles
                dlg = wx.lib.dialogs.ScrolledMessageDialog(self, msg,
                      "Could not load all items")
                dlg.ShowModal()
            if len(self.toRename) != 0:
                self.bottomWindow.go.Enable(True)
                self.menuFile.GoMenu.Enable(True)
                self.currentItem = None
                self.displayResults(True)
            f.close()


    # opens help dialog:
    def onMenuHelpHelpMenu(self, event):
        helpDiag.create(self).Show()

    # opens generic html help dialog:
    def showSmallHelp(self, event):
        if event.GetId() == wxID_MENUHELP_REHELP:
            helpFile=u'REhelp.html'
            Title=_(u"Regular Expression Help")
            Icon=u're'
        elif event.GetId() == wxID_MENUHELP_FORMATHELP:
            helpFile=u'format.html'
            Title=_(u"Date and Time Formatting Help")
            Icon=u'date_time'
        elif event.GetId() == wxID_MENUHELP_EXAMPLES:
            helpFile=u'examples.html'
            Title=_(u"Examples")
            Icon=u'examples'
        return classes.SmallHelp(self,self,helpFile,Title,Icon).Show()

    # opens about dialog:
    def onMenuHelpAboutMenu(self, event):
        aboutDlg = about.create(self)
        aboutDlg.CentreOnParent()
        aboutDlg.ShowModal()
        aboutDlg.Destroy()

    # opens preferences dialog:
    def onMenuEditPreferencesMenu(self, event):
        prefDiag = preferences.createDialog(self)
        prefDiag.ShowModal()
        prefDiag.Destroy()



#### DISPLAY RESULTS ###########################################################
    """
    Resets the bottom preview virtual list, then sets amount of items and the
    correct imagelist to use.
    """
    def displayResults(self, showDirs=False):
        self.errors.clearErrors()

        pickerList = self.picker.Panel.ItemList
        display = self.bottomWindow.display
        #display.Enable(False)
        usedOperations = self.renamer.Panel.usedOperations
        # show complete path in new name
        if not showDirs:
            for i in range(usedOperations.GetItemCount()):
                op = usedOperations.GetItemText(i)
                if re.search(_("directory"),op):
                    showDirs = True
                    break
        display.showDirs = showDirs
        display.mode = 'preview'

        # set the image list
        if self.prefs.get('showPreviewIcons'):
            display.imgs = pickerList.imgs
        else:
            display.imgs = wx.ImageList(16,16)
            display.folderIco = display.imgs.Add(wx.Bitmap(\
              utils.iconPath(u'folder16.png'),wx.BITMAP_TYPE_PNG))
            display.fileIco = display.imgs.Add(wx.Bitmap(\
              utils.iconPath(u'file16.png'),wx.BITMAP_TYPE_PNG))
        display.SetImageList(display.imgs, wx.IMAGE_LIST_SMALL)

        display.DeleteAllItems()
        display.SetItemCount(len(self.toRename))

        #display.Enable(True)

        # auto resize column:
        #display.SetColumnWidth(2,-1)
        if display.GetColumnWidth(2) < 135:
            display.SetColumnWidth(2,135)

        # show the last warning/error
        if self.warn != []:
            display.EnsureVisible(self.warn[-1])
        elif self.bad != []:
            display.EnsureVisible(self.bad[-1])

        # show the currently selected item
        try:
            display.Select(self.currentItem, True)
        except TypeError:#may need AttributeError
            pass
        else:
            display.EnsureVisible(self.currentItem)


#### CHANGE ITEM POSITION IN LIST ##############################################
    # set the selected item
    def onItemSelected(self, event):
        self.currentItem = event.m_itemIndex

    # move the selected item
    def changeItemOrder(self, change):
        if self.currentItem != None:
            try:
                moveTo = self.currentItem + change
            except TypeError:
                if change == u'top':
                    moveTo = 0
                else:
                    moveTo = len(self.items)-1
                pass

            if moveTo > len(self.items):
                moveTo = len(self.items)-1
            if moveTo < 0:
                moveTo = 0

            old = self.items[self.currentItem]
            new = self.items[moveTo]

            del self.items[self.currentItem]
            self.items.insert(moveTo,old)

            self.currentItem = moveTo
            self.onPreviewButton(0)


##### NAME GENERATION ##########################################################
    def onPreviewButton(self, event):
        utils.setBusy(True)
        self.renamer.previewItems(event)
        utils.setBusy(False)


##### RENAMING: ################################################################
    def renameItems(self, event):
        self.renamer.renameItems(event)


##### UNDO: ####################################################################
    def undoRename(self, event):
        self.renamer.undoRename(event)

#EOF
