# -*- coding: utf-8 -*-

# base classes used throughout application

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import os
import wx
import utils

# Load parameters from GUI to core
class Parameters:
    def setValueMethod(self):
        self.getValue = self.getWxValue

    # get a wxWidget value
    def getWxValue(self, widget):
        if isinstance(widget, wx.ComboBox):
            value = widget.GetStringSelection()
            # inconsistences in library requires this
            if not value:
                value = widget.GetValue()
        elif isinstance(widget, wx.Choice):
            value = widget.GetSelection()
        else:
            value = widget.GetValue()
        return value

    # set given widgets
    def setParameters(self, widgets):
        for widgetName in widgets:
            if hasattr(self.Panel, widgetName):
                widget = getattr(self.Panel, widgetName)
                setattr(self, widgetName, self.getValue(widget))
        return self


# display single page help dialog
class SmallHelp(wx.Dialog):
    def initCtrls(self, prnt, main, Title, Icon):
        # generated method, don't edit
        wx.Dialog.__init__(self, id=-1, name='smallHelpDialog', parent=prnt,
              size=wx.Size(610, 531),
              style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER | wx.MAXIMIZE_BOX, title=Title)
        self.SetIcon(wx.Icon(utils.realPath(u'icons/%s.ico')%Icon,wx.BITMAP_TYPE_ICO))

        self.display = wx.html.HtmlWindow(id=-1,
              name=u'display', parent=self, size=wx.Size(602,
              520), style=wx.html.HW_SCROLLBAR_AUTO)

    def __init__(self, parent, main, helpFile, Title, Icon):
        self.initCtrls(parent,main,Title,Icon)
        self.Center(wx.HORIZONTAL|wx.VERTICAL)
        if u'gtk2' in wx.PlatformInfo:
            self.display.SetStandardFonts()

        docspath = utils.realPath(u'help')

        if os.path.isfile(os.path.join(docspath,main.language,helpFile)):
            helpFile = os.path.join(docspath,main.language,helpFile)
        else:
            helpFile = os.path.join(docspath,u'en_US',helpFile)

        self.display.LoadPage(helpFile)


# progress dialog handling
class ProgressDialog(wx.ProgressDialog):
    def __init__(self, parent, prefs, items, message):
        
        if type(items) is not type(1):
            items = len(items)
        message = message.replace(u'%%%', unicode(items))

        if prefs.get('showProgressDialog') and items > int(prefs.get(u'itemCountForProgDialog')):
            self.active = True
            wx.ProgressDialog.__init__(self,
                                       _(u"Progress"),
                                       message,
                                       maximum = items,
                                       parent=parent,
                                       style =  wx.PD_APP_MODAL
                                        | wx.PD_ELAPSED_TIME
                                        | wx.PD_AUTO_HIDE
                                        | wx.PD_CAN_ABORT
                                       )
        else:
            self.active = False

    def update(self, c):
        if self.active:
            (keepGoing, skip) = self.Update(c)
            return keepGoing

    def destroy(self):
        if self.active:
            self.Destroy()
            self = False

