# -*- coding: utf-8 -*-

# This is the panel in charge of directory manipulations.

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import opButtons
import os
import re
import utils

[wxID_PANEL, wxID_PANELDIRECTORYTEXT, wxID_PANELRADIOBUTTON1,
 wxID_PANELRADIOBUTTON2, wxID_PANELSTATICTEXT1,
 wxID_PANELADDCURRENT, wxID_PANELSTATICTEXT2, wxID_PANELPATHRECUR,
 wxID_PANELINVERSE, wxID_PANELBROWSE, wxID_PANELSTATICTEXT3
] = [wx.NewId() for initCtrls in range(11)]

class Panel(wx.Panel):
    def sizer(self):
        superSizer = wx.BoxSizer(wx.VERTICAL)
        superSizer.Add(self.staticText1,0,wx.TOP|wx.BOTTOM|wx.LEFT,5)

        pathSizer = wx.BoxSizer(wx.HORIZONTAL)
        pathSizer.Add(self.browse,0,wx.TOP|wx.LEFT|wx.ALIGN_CENTER,5)
        pathSizer.Add(self.directoryText,1,wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT,5)
        superSizer.Add(pathSizer,0,wx.EXPAND)
        superSizer.Add(self.staticText3,1,wx.LEFT,100)
        superSizer.Add((-1,25),0)

        superSizer.Add(self.opButtonsPanel,0,wx.EXPAND)
        superSizer.Add((-1,15),0)

        absSizer = wx.BoxSizer(wx.HORIZONTAL)
        absSizer.Add(self.addCurrent,0,wx.ALIGN_CENTER|wx.LEFT,5)
        absSizer.Add(self.staticText2,0,wx.ALIGN_CENTER|wx.LEFT,15)
        absSizer.Add(self.pathRecur,0,wx.ALIGN_CENTER|wx.LEFT,5)
        absSizer.Add(self.inverse,0,wx.ALIGN_CENTER|wx.LEFT,10)
        superSizer.Add(absSizer,0,wx.BOTTOM,15)

        self.SetSizerAndFit(superSizer)


    def initCtrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_PANEL, name=u'directoryToolsPanel', parent=prnt,
              size=wx.Size(457, 334), style=wx.TAB_TRAVERSAL)

        self.opButtonsPanel = opButtons.Panel(self, main)

        self.staticText1 = wx.StaticText(id=wxID_PANELSTATICTEXT1,
              label=_(u"Name (absolute paths allowed):"), name='staticText1', parent=self,
              style=0)

        self.browse = wx.Button(id=wxID_PANELBROWSE,
              label=_(u"Browse"), name=u'browse', parent=self,
              style=0)
        self.browse.SetToolTipString(_(u"Browse for path"))
        self.browse.Bind(wx.EVT_BUTTON, self.browseForDir,
              id=wxID_PANELBROWSE)

        self.directoryText = wx.TextCtrl(id=wxID_PANELDIRECTORYTEXT,
              name=u'directoryText', parent=self, pos=wx.Point(32, 72),
              style=wx.TE_PROCESS_ENTER, value=u'')
        self.directoryText.Bind(wx.EVT_TEXT, self.correctPathType,
              id=wxID_PANELDIRECTORYTEXT)
        self.directoryText.Bind(wx.EVT_TEXT_ENTER, self.correctPathEnter,
                  id=wxID_PANELDIRECTORYTEXT)

        self.addCurrent = wx.Button(id=wxID_PANELADDCURRENT,
              label=_(u"Copy Path Structure"), name=u'addCurrent', parent=self,
              style=0)
        self.addCurrent.SetToolTipString(_(u"For best results, use with absolute paths."))
        self.addCurrent.Bind(wx.EVT_BUTTON, self.onAddCurrentButton,
              id=wxID_PANELADDCURRENT)

        self.staticText2 = wx.StaticText(id=wxID_PANELSTATICTEXT2,
              label=_(u"Path depth to copy:"), name='staticText2', parent=self,
              style=0)

        self.staticText3 = wx.StaticText(id=wxID_PANELSTATICTEXT3,
              label=_(u""),
              name='staticText3', parent=self,
              style=0)


        self.pathRecur = wx.SpinCtrl(id=wxID_PANELPATHRECUR, initial=1, max=255,
              min=-255, name=u'pathRecur', parent=self, size=wx.Size(65, -1),
              style=wx.SP_ARROW_KEYS|wx.TE_PROCESS_ENTER, value='1')
        self.pathRecur.SetValue(1)
        self.pathRecur.SetToolTipString(_(u"negative values allowed"))
        self.pathRecur.Bind(wx.EVT_TEXT_ENTER, main.showPreview,
              id=wxID_PANELPATHRECUR)
        self.pathRecur.Bind(wx.EVT_SPINCTRL, main.showPreview,
              id=wxID_PANELPATHRECUR)

        self.inverse = wx.CheckBox(id=wxID_PANELINVERSE, label=_(u"inverse"),
              name=u'inverse', parent=self, style=0)
        self.inverse.SetToolTipString(_(u"Start from begining or end of path"))
        self.inverse.SetValue(False)
        self.inverse.Bind(wx.EVT_CHECKBOX, main.showPreview)

    def __init__(self, parent, main_window):
        global main
        main = main_window
        self.initCtrls(parent)
        self.sizer()
        self.staticText3.Show(False)
        self.activatedField = self.directoryText
        # text is accessed by parent
        self.pathStructTxt = ':'+_(u"Path-Structure")+':'

    # add current directory
    def onAddCurrentButton(self, event):
        iPoint = self.directoryText.GetInsertionPoint()
        self.directoryText.WriteText(self.pathStructTxt)
        self.directoryText.SetFocus()
        self.directoryText.SetInsertionPoint(iPoint + len(self.pathStructTxt))

    # browse for directory
    def browseForDir(self, event):
        dlg = wx.DirDialog(self,_(u"Choose a directory:"),
            style=wx.DD_DEFAULT_STYLE)
        dlg.SetPath(self.directoryText.GetValue())
        try:
            if dlg.ShowModal() == wx.ID_OK:
                self.directoryText.SetValue('')
                dir = dlg.GetPath()
                iPoint = self.directoryText.GetInsertionPoint()
                self.directoryText.WriteText(dir)
                self.directoryText.SetFocus()
                self.directoryText.SetInsertionPoint(iPoint + len(dir))
        finally: dlg.Destroy()


    # --- CORRECT SOME COMMON MISTAKES --- #
    # preliminary, needs more work!

    #from http://regexlib.com
    """
    ^[^\\\./:\*\?\"<>\|]{1}[^\\/:\*\?\"<>\|]{0,254}$
    Validates a long filename using Windows' rules. Requires one
    valid filename character other than "." for the first
    character and then any number of valid filename characters up to a
    total length of 255 characters. Unresolved is how to prevent the
    last character from being a "." while still meeting all
    the features that this regex does now.
    -- Dale Preston
    """

    """
    ([a-zA-Z]:(\\w+)*\\[a-zA-Z0_9]+)?.xls
    This RegEx will help to validate a physical file path with a
    specific file extension (here xls)
    -- vinod kumar
    """

    """
    ^(([a-zA-Z]:|\\)\\)?(((\.)|(\.\.)|([^\\/:\*\?"\|<>\. ](([^\\/:\*\?"\|<>\. ])|([^\\/:\*\?"\|<>]*[^\\/:\*\?"\|<>\. ]))?))\\)*[^\\/:\*\?"\|<>\. ](([^\\/:\*\?"\|<>\. ])|([^\\/:\*\?"\|<>]*[^\\/:\*\?"\|<>\. ]))?$
    File Name Validator. Validates both UNC (\\server\share\file) and regular MS path (c:\file).
    -- Dmitry Borysov
    """


    # fix/detect separator issues
    def separatorFix(self):
        # remove dupe path seperators and make sure
        # only valid path seperators are used
        if os.sep == "/":
            pattern = "(\\\\{1,})|(\/{2,})"
            repl = "/"
        else:
            pattern = "(\\\\{2,})|(\/{1,})"
            repl = "\\\\"
        return pattern, repl

    # user presses enter, errors fixed
    def correctPathEnter(self, event):
        path = self.directoryText.GetValue()
        pattern, replacement = self.separatorFix()
        # fix
        newpath = re.sub(pattern,replacement,path)
        self.directoryText.SetValue(newpath)
        self.directoryText.SetInsertionPointEnd()
        self.staticText3.Show(False)
        # apply
        self.directoryText.SetBackgroundColour(u'#FFFFFF')
        self.directoryText.Refresh()
        main.showPreview(event)

    # user types, warning shown
    def correctPathType(self, event):
        path = self.directoryText.GetValue()
        pattern, replacement = self.separatorFix()
        # warn
        if re.search(pattern,path):
            err = _(u"Incorrect path separator")
            self.staticText3.SetLabel(_(u"%s - press ENTER to fix")%err)
            utils.resetMinSize(self.staticText3,0)
            self.directoryText.SetBackgroundColour(u"#FDEB22")
            self.directoryText.Refresh()
            self.staticText3.Show(True)
        # apply
        else:
            main.showPreview(event)



    # is the path structure an absolute path?
    """
    def checkAbs(self, event):
        path = self.directoryText.GetValue()
        absPathWidgets = (self.addCurrent, self.staticText2, self.pathRecur, self.inverse)
        if os.path.isabs(path):
            for widget in absPathWidgets:
                widget.Enable(True)
        else:
            for widget in absPathWidgets:
                widget.Enable(False)
        main.showPreview(event)
    """

