# -*- coding: utf-8 -*-

# base class for all operations
# TODO - a lot !

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import os
import utils

class Operation(wx.Panel):
    def __init__(self, params={}):
        # default parameters
        # need to define here due to python's handling of class instances
        self.params = {
            'applyPathOnly' : False,
            'applyName' : True,
            'applyExtension' : False,
        }
        # allow passed-in parameters to overide defaults or add additional
        # parameters.
        for k in params.keys():
            self.params[k] = params[k]

    # Operations that only modify the path (e.g. 'directory')
    def setAsPathOnly(self):
        self.params['applyPathOnly'] = True
        self.params['applyName'] = False
        self.params['applyExtension'] = False

    # join extensions
    def joinExt(self, name, ext):
        if self.params['applyName'] and not self.params['applyExtension']:
            newName = name
        elif self.params['applyExtension'] and not self.params['applyName']:
            newName = ext
        elif self.params['applyName'] and self.params['applyExtension']:
            if ext != '':
                newName = name+'.'+ext
            else:
                newName = name
        else:
            newName = False
        return newName

    # split extensions
    def splitExt(self, newName, name, ext):
        if self.params['applyName'] and not self.params['applyExtension']:
            name = newName
        elif self.params['applyExtension'] and not self.params['applyName']:
            ext = newName
        elif self.params['applyName'] and self.params['applyExtension']:
            name = os.path.splitext(newName)[0]
            ext = os.path.splitext(newName)[1][1:]
        return name,ext

    # Create a notebook for operations that require numbering and date & time settings
    # (directory, insert, replace)
    def initNotebook(self, main, parent, varPanel, varText, varIcon):
        # generated method, don't edit
        parent.AddPage(imageId=-1, page=varPanel, select=True,
              text=varText)
        parent.AddPage(imageId=-1, page=self.numberingPanel, select=False,
              text=_(u"Numbering settings"))
        parent.AddPage(imageId=-1, page=self.dateTimePanel, select=False,
              text=_(u"Date && Time settings"))
        # load and assign notebook page images
        il = wx.ImageList(16, 16)
        img0 = il.Add(wx.Bitmap(utils.iconPath(varIcon), wx.BITMAP_TYPE_ICO))
        img1 = il.Add(wx.Bitmap(utils.iconPath(u'numbering.ico'), wx.BITMAP_TYPE_ICO))
        img2 = il.Add(wx.Bitmap(utils.iconPath(u'date_time.ico'), wx.BITMAP_TYPE_ICO))
        parent.AssignImageList(il)
        parent.SetPageImage(0, img0)
        parent.SetPageImage(1, img1)
        parent.SetPageImage(2, img2)
        # required to make XP theme better:
        self.notebook.SetBackgroundColour(main.notebook.GetThemeBackgroundColour())
