# -*- coding: utf-8 -*-

# Panel to hold regular expression stuff

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import re
import sre_constants

[wxID_PANEL, wxID_PANELA_Z, wxID_PANELDIGIT,
 wxID_PANELREG_EXPR, wxID_PANELREG_EXP_DIV,
 wxID_PANELREG_EXP_I, wxID_PANELREG_EXP_U,
] = [wx.NewId() for initCtrls in range(7)]

class Panel(wx.Panel):
    def sizer(self):
        regExprSizer = wx.BoxSizer(wx.HORIZONTAL)
        regExprSizer.Add(self.reg_expr,0,wx.ALIGN_CENTER|wx.RIGHT,3)
        regExprSizer.Add(self.reg_exp_i,0,wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT,3)
        regExprSizer.Add(self.reg_exp_u,0,wx.ALIGN_CENTER)
        regExprSizer.Add((0,0),1)
        regExprSizer.Add(self.reg_exp_div,0,wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT,3)
        regExprSizer.Add((0,0),1)
        regExprSizer.Add(self.a_z,0,wx.ALIGN_CENTER|wx.RIGHT,5)
        regExprSizer.Add(self.digit,0,wx.ALIGN_CENTER)
        regExprSizer.Add((0,0),60)
        self.SetSizerAndFit(regExprSizer)
        #mainSizer.Add(regExprSizer,0,wx.EXPAND|wx.TOP,5)

    def initCtrls(self, prnt, Name):
        wx.Panel.__init__(self, id=wxID_PANEL, name=Name, parent=prnt,
              size=wx.Size(457, 334), style=wx.TAB_TRAVERSAL)

        self.reg_expr = wx.CheckBox(id=wxID_PANELREG_EXPR,
              label=_(u"Evaluate as regular expression"), name=u'reg_expr', parent=self,
              style=0)
        self.reg_expr.SetValue(False)
        self.reg_expr.Bind(wx.EVT_CHECKBOX, prnt.activateOptions,
              id=wxID_PANELREG_EXPR)

        self.reg_exp_i = wx.CheckBox(id=wxID_PANELREG_EXP_I, label=_(u"case sensitive"),
              name=u'reg_exp_i', parent=self)
        self.reg_exp_i.SetValue(False)
        self.reg_exp_i.SetToolTipString(_(u"Differentiate between upper and lower case"))
        self.reg_exp_i.Enable(False)
        self.reg_exp_i.Bind(wx.EVT_CHECKBOX, prnt.defineRegExp)

        self.reg_exp_u = wx.CheckBox(id=wxID_PANELREG_EXP_U, label=_(u"Unicode"),
              name=u'reg_exp_u', parent=self)
        self.reg_exp_u.SetValue(True)
        self.reg_exp_u.SetToolTipString(_(u"Unicode match (\w matches 'a','b','c', etc)"))
        self.reg_exp_u.Enable(False)
        self.reg_exp_u.Bind(wx.EVT_CHECKBOX, prnt.defineRegExp)

        self.reg_exp_div = wx.StaticLine(id=wxID_PANELREG_EXP_DIV,
              name=u'reg_exp_div', parent=self, size=wx.Size(2, 21),
              style=wx.LI_VERTICAL)

        self.a_z = wx.Button(id=wxID_PANELA_Z, label=_(u"[a-z]"), name=u'a_z',
              parent=self, style=wx.BU_EXACTFIT)
        self.a_z.SetToolTipString(_(u"All alphabetical characters"))
        self.a_z.Enable(False)
        self.a_z.Bind(wx.EVT_BUTTON, self.addReButton)

        self.digit = wx.Button(id=wxID_PANELDIGIT, label=_(u"[0-9]"),
              name=u'digit', parent=self, style=wx.BU_EXACTFIT)
        self.digit.SetToolTipString(_(u"All number characters"))
        self.digit.Enable(False)
        self.digit.Bind(wx.EVT_BUTTON, self.addReButton)

    def __init__(self, parent, main_window, name=u'regExpPanel'):
        global main
        main = main_window
        self.parent = parent
        self.initCtrls(parent, name)
        self.sizer()

    # set active widgets
    def setActive(self, activate):
        reg_exp_tup = (self.reg_exp_i, self.reg_exp_u,
          self.a_z,self.digit)
        for option in reg_exp_tup:
            option.Enable(activate)

    # add some predefined REs
    def addReButton(self, event):
        if event.GetId() == wxID_PANELA_Z:
            txt = "[^\W\d]"
        elif event.GetId() == wxID_PANELDIGIT:
            txt = "\d"
        activatedField = self.activatedField

        iPoint = activatedField.GetInsertionPoint()
        activatedField.WriteText(txt)
        activatedField.SetFocus()
        activatedField.SetInsertionPoint(iPoint + len(txt))


    # find focused text field
    def setActivatedField(self, event):
        self.activatedField = event.GetEventObject()


    # compile regular expression
    def createRE(self, REfind):
        self.parent.REmsg = False
        caseSensitive = self.reg_exp_i.GetValue()
        useUnicode = self.reg_exp_u.GetValue()
        try:
            # compile according to options
            if not caseSensitive and useUnicode:
                mod = re.compile(REfind, re.IGNORECASE | re.UNICODE)
            elif not caseSensitive:
                mod = re.compile(REfind, re.IGNORECASE)
            elif useUnicode:
                mod = re.compile(REfind, re.UNICODE)
            else:
                mod = re.compile(REfind)
        except sre_constants.error as err:
            # make blank regular expression
            mod = re.compile(r'')

            # give user a descriptive error message
            # set message directly for when items are NOT loaded
            main.setStatusMsg(_(u"Regular-Expression: %s")%err,u'warn')

            # set message in variable for when items ARE loaded
            self.parent.REmsg = _(u"Regular-Expression: %s")%err

            # so we know not to change status text after re error msg
            main.REmsg = True
            pass
        else:
            # reset status message for when items are not loaded
            main.setStatusMsg("",u'eyes')
            return mod

