# -*- coding: utf-8 -*-

# This is the panel in charge of replacing matches with text or operations.

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
from operation import Operation
import replaceTools
import numbering
import dateTime
import utils
import sre_constants

[wxID_PANEL, wxID_PANELNOTEBOOK] = [wx.NewId() for initCtrls in range(2)]

class OpPanel(Operation):
    def sizer(self, parent):
        #smallestSize = parent.rightSizer.GetSize() - parent.rightTopSizer.GetSize() - (10,10)
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        #mainSizer.SetMinSize(smallestSize)
        mainSizer.Add(self.notebook,0,wx.EXPAND)
        self.SetSizerAndFit(mainSizer)

    def initReplaceNotebook(self, parent):
        self.initNotebook(main, parent, self.ReplaceToolsPanel,
                          _(u"Replace settings"), u'replace.ico')

    def initCtrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_PANEL, name=u'Panel', parent=prnt,
              style=wx.TAB_TRAVERSAL)

        self.notebook = wx.Notebook(id=wxID_PANELNOTEBOOK,
              name=u'notebook', parent=self, style=wx.NB_BOTTOM|wx.NO_BORDER)
        self.notebook.SetThemeEnabled(True)
        # notebook pages
        self.ReplaceToolsPanel = replaceTools.Panel(self.notebook, main)
        self.numberingPanel = numbering.Panel(self.notebook, main)
        self.dateTimePanel = dateTime.Panel(self.notebook, main)


    def __init__(self, parent, main_window, params={}):
        Operation.__init__(self, params)
        global main
        main = main_window
        self.initCtrls(parent)
        self.initReplaceNotebook(self.notebook)
        self.sizer(parent)

    # update GUI elements, settings after config load
    def onConfigLoad(self):
        self.ReplaceToolsPanel.onLoad()
        self.numberingPanel.onConfigLoad()
        self.dateTimePanel.getFromItemCheckbox(False)

    # used to reset the numbering counter for the operation
    def resetCounter(self, c):
        utils.resetCounter(self, self.ReplaceToolsPanel, c)

    def renameItem(self, path, name, ext, original):
        operations = self.ReplaceToolsPanel.opButtonsPanel
        newName = self.joinExt(name,ext)
        if not newName:
            return path,name,ext

        # basic settings
        search = self.notebook.GetPage(0).search
        searchValues = search.searchValues
        text = self.notebook.GetPage(0).repl_txt.GetValue()

        #- do nothing
        if searchValues[0] == u"text" and not searchValues[2] and not text:
            return path,name,ext

        #- search and replace when field not blank:
        elif searchValues[0] == u"text" and searchValues[2]:
            find = searchValues[2]
            parsedText = operations.parseInput(text, original, self)
            #case insensitive
            if not searchValues[1]:
                found = search.caseInsensitive(newName)
                for match in found:
                    newName = newName.replace(match, parsedText)
            #case sensitive:
            else:
                newName = newName.replace(find, parsedText)
        
        #- replace everything if text field is blank:
        elif searchValues[0] == u"text":
            newName = operations.parseInput(text, original, self)

        #- replace using regular expression:
        elif searchValues[0] == u"reg-exp" and searchValues[1]:
            # need to first substiute, then parse for backreference support
            try:
                replaced = searchValues[2].sub(text, newName)
            except (sre_constants.error,AttributeError) as err:
                main.setStatusMsg(_(u"Regular-Expression: %s")%err,u'warn')
                # so we know not to change status text after RE error msg:
                main.REmsg = True
                pass
            else:
                newName = operations.parseInput(replaced, original, self)

            # show RE error message from search, if any
            if search.REmsg:
                main.setStatusMsg(search.REmsg,u'warn')
                main.REmsg = True

        #- replace in between
        elif searchValues[0] == u"between":
            positions = search.getBetweenToFrom(newName)
            if positions:
                frm = positions[0]
                to = positions[1]
                parsedText = operations.parseInput(text,original,self)
                newName = newName[:frm] + parsedText + newName[to:]

        #- replace by position:
        elif searchValues[0] == u"position":
            ss = search.repl_from.GetValue()
            sl = search.repl_len.GetValue()
            frm,to,tail = search.getStartEndPos(ss,sl,newName)
            parsedText = operations.parseInput(text,original,self)
            newName = newName[:frm] + parsedText + tail

        name,ext = self.splitExt(newName,name,ext)
        return path,name,ext