# -*- coding: utf-8 -*-

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

from __future__ import print_function
import wx
import os
import codecs
import utils
import general
import automation
import display
import errorCheck
import logging

def createDialog(parent, initial=False):
    return Dialog(parent,initial)

[wxID_DIALOG, wxID_DIALOGAPPLY, wxID_DIALOGCLOSE, wxID_DIALOGNOTEBOOK,
 wxID_DIALOGOK,
] = [wx.NewId() for initCtrls in range(5)]

class Methods:
    def __init__(self, MainWindow=False):
        global main
        main = MainWindow
        self.header = u'Version=%s\n'%main.version
        self.prefs = False

    def findPrefFile(self):
        prefFile = utils.stdPath(u'preferences.ini')
        return prefFile
    
    def openPrefFile(self,type):
        prefFile = self.findPrefFile()
        utils.debugPrint(main, "Opening as '%s' : %s"%(type,prefFile))
        prefFile = codecs.open(prefFile,type, 'utf-8')
        return prefFile
        
    # get values from panels and set to file
    def setPrefs(self, prefDialog):
        prefFile = self.openPrefFile('w')
        options = self.header
        # get all pages in notebook
        for i in range(prefDialog.notebook.GetPageCount()):
            # section header is page name
            options += u'\n[%s]\n'%prefDialog.notebook.GetPageText(i)

            # get all controls in page
            for child in prefDialog.notebook.GetPage(i).GetChildren():
                value = None
                # order of instance checks is important
                if isinstance(child, wx.CheckBox):
                    value = int(child.GetValue())
                    type = 'CheckBox'
                elif isinstance(child, wx.RadioButton):
                    value = int(child.GetValue())
                    type = 'RadioButton'
                elif isinstance(child, wx.TextCtrl):
                    value = child.GetValue()
                    type = 'TextCtrl'
                elif isinstance(child, wx.SpinCtrl):
                    value = child.GetValue()
                    type = 'SpinCtrl'
                elif isinstance(child, wx.ComboBox):
                    value = child.GetValue()
                    type = 'ComboBox'
                elif isinstance(child, wx.Choice):
                    value = int(child.GetSelection())
                    type = 'Choice'
                
                if value is not None:
                    name = child.GetName()
                    options += u'%s=%s\n'%(name,value)
                    utils.debugPrint(main, "%s (%s) = %s"%(name,type,value))
        prefFile.write(options)


    # create default preferences file
    def createNew(self):
        prefFile = self.openPrefFile('w+')
        msg = _(u"Please take a moment to set your preferences.\n\n")
        title = _(u"Preferences")
        dlg = wx.MessageDialog(None, msg, title, wx.CAPTION|wx.OK|wx.ICON_EXCLAMATION)
        dlg.ShowModal()
        dlg.Destroy()

        # show preferences
        prefDiag = Dialog(main, initial=True)
        prefDiag.ShowModal()
        prefDiag.Destroy()

        return self.readFile(prefFile)


    # process file and assign results to prefs dictionary:
    def readFile(self, prefFile):
        prefFile.seek(0)
        prefs = {}
        for line in prefFile:
            if line.find('=') > 0:
                line = line.replace(u'\n','')
                split = line.split('=')
                try:
                    prefs[split[0]] = int(split[1])
                except KeyError:
                    pass
                except ValueError:
                    prefs[split[0]] = unicode(split[1])
                    pass
        return prefs

    # get values from file (one way or another)
    def loadPreferences(self):
        prefFile = self.findPrefFile()
        # make sure the file exist and is filled:
        if not os.path.exists(prefFile) or os.path.getsize(prefFile) < 5:
            prefs = self.createNew()
        else:
            prefFile = self.openPrefFile('r')
            prefFile.seek(0)
            version = prefFile.readline().strip()[8:]
            # preferences are from prior version, create new file
            if version[:4] != main.version[:4]:
                prefs = self.createNew()
            else:
                prefs = self.readFile(prefFile)

        # use windows-compatible filenames?
        if prefs[u'useWinChars']:
            prefs[u'bad_chars'] = (u'\\',u'/',u':',u'*',u'?',u'"',u'>',u'<',u'|')

        if prefs[u'useWinNames']:
            prefs[u'bad_win_words'] = (u'con', u'prn', u'aux', u'clock$',
              u'nul', u'com1', u'com2', u'com3', u'com4', u'com5', u'com6', u'com7',
              u'com8', u'com9', u'lpt1', u'lpt2', u'lpt3', u'lpt4', u'lpt5',u'lpt6',
              u'lpt7', u'lpt8', u'lpt9')
        self.prefs = prefs

    # attempt to load a preference setting, recreate pref file if needed
    def get(self, preference, strict=True):
        if not self.prefs:
            self.loadPreferences()
        try:
            return self.prefs[preference]
        except KeyError:
            if not strict:
                pass
            else:
                msg = _(u"\nThe preferences file is outdated or corrupt.")
                msg += _(u"\nWill now recreate the preferences file.")
                utils.makeErrMsg(msg, _(u"Problem with preferences"))

                self.createNew()
                self.loadPreferences()
                return self.get(preference)


class Dialog(wx.Dialog):
    def initCollMainSizerItems(self, parent):
        # generated method, don't edit

        parent.AddWindow(self.notebook, 1, border=5,
              flag=wx.ALL | wx.EXPAND)
        parent.AddSizer(self.buttons, 0, border=5, flag=wx.ALL | wx.EXPAND)

    def initCollButtonsItems(self, parent):
        # generated method, don't edit

        parent.AddWindow(self.apply, 0, border=20, flag=wx.RIGHT)
        parent.AddWindow(self.ok, 0, border=105, flag=wx.RIGHT)
        parent.AddSpacer((10,8), 1, border=0, flag=0)
        parent.AddWindow(self.close, 0, border=0, flag=0)

    def initSizers(self):
        # generated method, don't edit
        self.mainSizer = wx.BoxSizer(orient=wx.VERTICAL)

        self.buttons = wx.BoxSizer(orient=wx.HORIZONTAL)

        self.initCollMainSizerItems(self.mainSizer)
        self.initCollButtonsItems(self.buttons)

        self.SetSizer(self.mainSizer)

    def initCtrls(self, prnt):
        # generated method, don't edit
        wx.Dialog.__init__(self, id=wxID_DIALOG, name=u'dialog', parent=prnt,
              style=wx.CAPTION | wx.RESIZE_BORDER, title=u'Preferences')

        self.notebook = wx.Notebook(id=wxID_DIALOGNOTEBOOK, name=u'notebook',
              parent=self, pos=wx.Point(5, 5), size=wx.Size(590, 40), style=0)

        self.apply = wx.Button(id=wx.ID_APPLY, name=u'apply', parent=self, style=0)
        self.apply.Bind(wx.EVT_BUTTON, self.onApplyButton)

        self.ok = wx.Button(id=wx.ID_OK, name=u'ok', parent=self, style=0)
        self.ok.Bind(wx.EVT_BUTTON, self.onOKbutton)

        self.close = wx.Button(id=wx.ID_CANCEL, name=u'close', parent=self, style=0)
        self.close.Bind(wx.EVT_BUTTON, self.closeDiag)

        self.initSizers()

    def __init__(self, parent, initial):
        global main
        main = parent
        self.initial = initial
        self.initCtrls(parent)
        self.SetIcon(wx.Icon(utils.iconPath(u'preferences.ico'),
                     wx.BITMAP_TYPE_ICO))
        self.notebook.SetBackgroundColour(self.notebook.GetThemeBackgroundColour())


        self.prefMethods = Methods(main)
        if not initial:
            prefs = self.prefMethods
        else:
            self.close.Enable(False)
            self.apply.Enable(False)

        # the panels to add to the notebook
        self.panels = ((general,_(u'General')),
                       (display, _(u'Display')),
                       (automation, _(u'Automate')),
                       (logging, _(u'Logging')),
                       (errorCheck, _(u'Error Checks')))
        # add panel to notebook
        i = 0
        for pane in self.panels:
            page = getattr(pane[0], 'Panel')
            self.notebook.AddPage(page(self.notebook), pane[1])
            notebookPage = self.notebook.GetPage(i)
            if not initial:
                self.loadPrefs(notebookPage, prefs)
            if hasattr(notebookPage, 'initEnabled'):
                notebookPage.initEnabled()
            i += 1

        #utils.resetMinSize(self)
        utils.resetMinSize(self,ignoreClasses=(wx.TextCtrl,wx.Button,wx.Choice,
                                               wx.SpinCtrl))
        # set the window size
        self.Fit()
        self.CentreOnScreen()

        # to see if these change when applying
        if not initial:
            self.oldDirTree = prefs.get(u'useDirTree')
            self.oldSHowHiddenDirs = prefs.get(u'showHiddenDirs')


    def onApplyButton(self, event):
        self.prefMethods.setPrefs(self)
        prefs = main.prefs = self.prefMethods

        #if self.prefs.get(u'autoPreviewPicker'):
        #    self.OnPreviewButton(0)
        if not self.initial:
            if prefs.get(u'useDirTree') != self.oldDirTree:
                main.notebook.GetPage(0).setTree()
            if prefs.get(u'showHiddenDirs') != self.oldSHowHiddenDirs:
                main.notebook.GetPage(0).dirPicker.ShowHidden(prefs.get(u'showHiddenDirs'))

            self.oldDirTree = prefs.get(u'useDirTree')
            self.oldSHowHiddenDirs = prefs.get(u'showHiddenDirs')

            main.showPreview(event)


    # load preferences from file and apply them to all panels
    def loadPrefs(self, panel, prefs):
        utils.debugPrint(main, 'Loading %s preferences ...'%panel.GetName())
        for child in panel.GetChildren():
            try:
                v = prefs.get(child.GetName(), False)
            except KeyError:
                pass
            else:
                utils.debugPrint(main, "   %s = %s"%(child.GetName(), v))
                
                if isinstance(child, wx.CheckBox) or isinstance(child, wx.RadioButton)\
                  or isinstance(child, wx.SpinCtrl):
                    child.SetValue(v)
                elif isinstance(child, wx.ComboBox) or isinstance(child, wx.TextCtrl):
                    child.SetValue(unicode(v))
                elif isinstance(child, wx.Choice):
                    child.SetSelection(v)


    def closeDiag(self, event):
        self.Close()

    def onOKbutton(self, event):
        self.onApplyButton(event)
        self.Close()
