# -*- coding: utf-8 -*-

# Helping functions

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

from __future__ import print_function
import wx
import wx.html
import os
import sys
import codecs
import locale
import time
try:
    import Image
except:
    pass


# Color definitions (get from system)
try:
    BackgroundClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW)
    HighlightClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT)
    HighlightTxtClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT)
    TxtClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOWTEXT)
except:
    pass


# get version info from file
def getVersion():
    try:
        f = open(realPath("version"))
    except:
        v = 'unknown'
    else:
        v = f.readline().strip()
        f.close()
    return v + u' (beta)'


# get wxPython Version
def getWxVersion():
    wxVer = ""
    for x in wx.VERSION:
        wxVer = wxVer + str(x) + "."
    wxVer = wxVer.rstrip(".")
    return wxVer


# determine if PIL is loaded
def pilIsLoaded():
    try:
        Image
    except NameError:
        return False
    else:
        return True


def debugPrint(main, msg):
    if main.debug:
        try:
            print(msg)
        except:
            print('invalid char in msg')


# reset all sizes to platform defaults (modified from Boa-Constructor)
def resetMinSize(parent, ignoreCtrls=(), ignoreClasses=()):
    # need this for some cases in linux, otherwise cuts off text
    if wx.Platform == '__WXGTK__' and isinstance(parent, wx.StaticText):
        textSize = parent.GetTextExtent(parent.GetLabel())
        size = wx.Size(textSize[0]+2,-1)
    else:
        size = wx.DefaultSize

    parent.SetMinSize(size)
    parent.SetSize(size)

    for child in parent.GetChildren():
        if child not in ignoreCtrls and not isinstance(child, ignoreClasses):
            resetMinSize(child, ignoreCtrls, ignoreClasses)


# adjust button sizes when using BU_EXACTFIT
# ignore is a tuple of buttons to ignore
def adjustExactbuttons(parent, ignore=()):
    for child in parent.GetChildren():
        if isinstance(child, wx.Button) and child.GetName() not in ignore:
            txt = child.GetLabel()
            size = wx.Size(child.GetTextExtent(txt)[0] + 18,-1)
            child.SetMinSize(size)


# create necessary folders
def makeInitialEnvironment():
    if not os.path.exists(stdPath('undo')):
        try:
            os.makedirs(stdPath('undo'))
        except OSError as error:
            if error[0] == 17:
                pass
            else:
                makeErrMsg(unicode(error), u"Error")
                pass


# Calculate the correct start and end positions for slices
def calcStartEndPos(s,l):
    if s == -1 and l == 1:
        frm = -1
        to = None
    elif s < 0 and l > 1:
        frm = s#-l+1
        to = s+l
        if to >= 0:
            to = None
    else:
        frm = s
        to = s+l
    return frm,to


# generic yes / no choice message
def makeYesNoMsg(msg, title):
    dlg = wx.MessageDialog(None, msg, title, wx.YES_NO | wx.YES_DEFAULT | wx.ICON_WARNING)
    if dlg.ShowModal() == wx.ID_YES:
        returnValue = True
    else:
        returnValue = False
    dlg.Destroy()
    return returnValue

# Generic error message
def makeErrMsg(msg, title):
    dlg = wx.MessageDialog(None, msg, title, wx.ICON_ERROR)
    dlg.ShowModal()
    dlg.Destroy()

# Generic error message
def makeWarnMsg(msg, title):
    dlg = wx.MessageDialog(None, msg, title, wx.ICON_WARNING)
    dlg.ShowModal()
    dlg.Destroy()

# return user path
def stdPath(file):
    base = wx.StandardPaths.Get().GetUserConfigDir()
    return os.path.join(base,u'.metamorphose2',file)


# return application path
def realPath(file):
    if hasattr(sys, "frozen"):
        path = os.path.dirname(sys.executable)
    else:
        path = False
        for path in sys.path:
            if 'metamorphose' in path:
                path = path.decode(sys.getfilesystemencoding())
                break
        if not path:
            print("Could not determine application path.\nMake sure the application is correctly installed.\n")
            sys.exit();
        #print(path)
    return os.path.join(path,file)

# return locale directory
def localePath(lang):
    # windows py2exe
    if hasattr(sys, "frozen"):
        return realPath(u'messages')
    # Linux, freeBSD when installed
    elif os.path.exists(u'/usr/share/locale/%s/LC_MESSAGES/metamorphose2.mo'%lang):
        return u'/usr/share/locale'
    # run from source
    else:
        return realPath(u'../messages')

# return full icon path
def iconPath(icon):
    return realPath(u'icons/%s'%icon)

# add item to warnings
def addToWarnings(main,path,msg):
    ec = main.ec
    if main.warn.count(ec) < 1:
        main.warn.append(ec)
        main.errorLog.insert(0, (ec, path, msg, u'warn'))

# add item to errors
def addToErrors(main, path, msg):
    ec = main.ec
    if main.bad.count(ec) < 1:
        main.bad.append(ec)
        main.errorLog.insert(0, (ec, path, msg, u'bad'))


# reset a numbering counter
def resetCounter(parent, Tools, c):
    Tools.opButtonsPanel.counter = c
    Tools.opButtonsPanel.auxCount = 0
    parent.numberingPanel.setNumberParams(True)


# write a file
def writeFile(path, data):
    try:
        os.makedirs(os.path.dirname(path))
    except:
        pass
    try:
        fp = codecs.open(path, 'w', 'utf_8')
    except (IOError, OSError) as error:
        makeErrMsg(unicode(error), _(u"Error"))
        pass
    else:
        fp.write(unicode(data))
        fp.close()


# get all notebook page names
def getNotebookPages(parent):
    pages = []
    for i in range(parent.notebook.GetPageCount()):
        pages.append(parent.notebook.GetPage(i))
    return pages

# sometimes need to bind to this when programmatically doing things
def devNull(event):
    pass

# Time encode/decode crap, a workaround for strftime not accepting unicode
def uDate(main, format, itemDateTime):
    #print(main.encoding)
    udate = time.strftime(format.encode(main.encoding), itemDateTime)
    return udate.decode(main.encoding)

def setBusy(busy):
    if busy:
        wx.BeginBusyCursor()
    else:
        wx.EndBusyCursor()

# encodings lookup
def getEncodingChoices(key):
    encodingLookup = {
        _(u"Use System Default") : [u"%s"%locale.getlocale()[1], ],
        _(u'All Languages / Unicode') : [u'utf_8', u'utf_16'],
        _(u'Western Europe') : [u'windows-1252', u'iso-8859-1', u'iso-8859-15',
            u'mac_roman', u'cp500', u'cp850', u'cp1140'],
        _(u'Central & Eastern Europe') : [u'windows-1250', u'iso-8859-2',
            u'mac_latin2', u'cp852'],
        _(u'Esperanto, Maltese') : [u'iso-8859-3', ],
        _(u'Nordic Languages') : [u'iso-8859-10', u'mac_iceland', u'cp861',
            u'cp865'],
        _(u'Celtic Languages') : [u'iso8859_14', ],
        _(u'Baltic Languages') : [u'windows-1257',u'iso-8859-13',u'cp775'],
        _(u'Cyrillic Languages') : [u'windows-1251',u'iso-8859-5',u'koi8_r',
            'koi8_u',u'mac_cyrillic',u'cp154',u'cp866',u'cp855'],
        _(u'Greek') : [u'windows-1253',u'iso-8859-7',u'mac_greek',u'cp737',
            u'cp869',u'cp875'],
        _(u'Turkish') : [u'windows-1254', u'iso-8859-9', u'mac_turkish',
            u'cp857', u'cp1026'],
        _(u'Hebrew') : [u'windows-1255', u'iso-8859-8', u'cp424', u'cp856',
            u'cp862'],
        _(u'Arabic') : [u'windows-1256', u'iso-8859-6', u'cp864'],
        _(u'Urdu') : [u'cp1006', ],
        _(u'Thai') : [u'cp874', ],
        _(u'Vietnamese') : [u'windows-1258', ],
        _(u'Traditional Chinese') : [u'ms950', u'big5hkscs', u'big5'],
        _(u'Simplified Chinese') : [u'gb2312', u'hz'],
        _(u'Unified Chinese') : [u'ms936', u'gb18030'],
        _(u'Korean') : [u'ms949', u'iso-2022-kr', u'ms1361', u'euc_kr'],
        _(u'Japanese') : [u'ms-kanji', u'iso-2022-jp', u'shift_jis', u'euc_jp',
            u'iso-2022-jp-1', u'iso-2022-jp-2004', u'iso-2022-jp-3',
            u'iso-2022-jp-ext', u'shift_jis_2004', u'shift_jisx0213',
            u'euc_jis_2004', u'euc_jisx0213'],
        }
    return encodingLookup[key]
